const uri = require('./uriGenerators');
const autoBind = require('../../util/autobind');

/**
 * Contacts
 */
class Contacts {
  constructor(http) {
    this.http = http;
    autoBind(this);
  }

  /**
   * @param {string} directoryId
   * @param {string} contactId
   * @return {Promise}
   */
  getContactIncentivePoints(directoryId, contactId) {
    return this.http.get(uri._incentivePointsUri(directoryId, contactId));
  }

  /**
   * @param {string} directoryId
   * @param {string} contactId
   * @param {string} key
   * @param {Object} bodyParams
   * @return {Promise}
   */
  adjustContactIncentivePoints(directoryId, contactId, key, bodyParams) {
    const headers = {'X-IDEMPOTENCY-KEY': key};
    return this.http.fetchJsonWithAuth(uri._incentivePointsUri(directoryId, contactId), {
      method: 'POST',
      headers: headers,
      body: JSON.stringify(bodyParams),
    });
  }

  /**
   * @param {string} directoryId
   * @param {string} mailingListId
   * @param {Object} queryParams
   * @return {Promise}
   */
  listContactsInMailingList(directoryId, mailingListId, queryParams) {
    return this.http.get(uri._mailingListsContactsUri(directoryId, mailingListId, ''), queryParams);
  }

  /**
   * @param {string} directoryId
   * @param {string} mailingListId
   * @param {string} contactId
   * @return {Promise}
   */
  getContactInMailingList(directoryId, mailingListId, contactId) {
    return this.http.get(uri._mailingListsContactsUri(directoryId, mailingListId, contactId));
  }

  /**
   * @param {string} directoryId
   * @param {string} mailingListId
   * @param {string} contactId
   * @param {Object} queryParams
   * @return {Promise}
   */
  getContactHistoryInMailingList(directoryId, mailingListId, contactId, queryParams) {
    return this.http.get(uri._mailingListsContactsHistoryUri(directoryId, mailingListId, contactId), queryParams);
  }

  /**
   * @param {string} directoryId
   * @param {string} mailingListId
   * @param {Object} bodyParams
   * @return {Promise}
   */
  createContactInMailingList(directoryId, mailingListId, bodyParams) {
    return this.http.post(uri._mailingListsContactsUri(directoryId, mailingListId, ''), bodyParams);
  }

  /**
   * @param {string} directoryId
   * @param {string} mailingListId
   * @param {string} contactId
   * @param {Object} bodyParams
   * @return {Promise}
   */
  updateContactInMailingList(directoryId, mailingListId, contactId, bodyParams) {
    return this.http.put(uri._mailingListsContactsUri(directoryId, mailingListId, contactId), bodyParams);
  }

  /**
   * @param  {string} directoryId
   * @param  {string} mailingListId
   * @param  {string} contactId
   * @return {Promise}
   */
  deleteContactInMailingList(directoryId, mailingListId, contactId) {
    return this.http.delete(uri._mailingListsContactsUri(directoryId, mailingListId, contactId));
  }

  /**
   * @param {string} directoryId
   * @param {string} contactId
   * @param {Object} queryParams
   * @return {Promise}
   */
  listContactTransactions(directoryId, contactId, queryParams) {
    return this.http.get(uri._contactsTransactionsUri(directoryId, contactId, ''), queryParams);
  }

  /**
   * @param {string} directoryId
   * @param {string} transactionId
   * @return {Promise}
   */
  getContactTransaction(directoryId, transactionId) {
    return this.http.get(uri._directoryTransactionsUri(directoryId, transactionId));
  }

  /**
   * @param {string} directoryId
   * @param {Object} bodyParams
   * @return {Promise}
   */
  createContactTransactions(directoryId, bodyParams) {
    return this.http.post(uri._directoryTransactionsUri(directoryId, ''), bodyParams);
  }

  /**
   * @param {string} directoryId
   * @param {string} transactionId
   * @param {Object} bodyParams
   * @return {Promise}
   */
  appendContactTransactions(directoryId, transactionId, bodyParams) {
    return this.http.post(uri._directoryTransactionsUri(directoryId, transactionId), bodyParams);
  }

  /**
   * @param {string} directoryId
   * @param {string} transactionId
   * @param {Object} bodyParams
   * @return {Promise}
   */
  updateContactTransaction(directoryId, transactionId, bodyParams) {
    return this.http.put(uri._directoryTransactionsUri(directoryId, transactionId), bodyParams);
  }

  /**
   * @param {string} directoryId
   * @param {string} transactionId
   * @return {Promise}
   */
  deleteContactTransaction(directoryId, transactionId) {
    return this.http.delete(uri._directoryTransactionsUri(directoryId, transactionId));
  }

  /**
   * @param {string} directoryId
   * @param {Object} queryParams
   * @return {Promise}
   */
  listDirectoryContacts(directoryId, queryParams) {
    return this.http.get(uri._contactsUri(directoryId, ''), queryParams);
  }

  /**
   * @param {string} directoryId
   * @param {string} contactId
   * @return {Promise}
   */
  getDirectoryContact(directoryId, contactId) {
    return this.http.get(uri._contactsUri(directoryId, contactId));
  }

  /**
   * @param {string} directoryId
   * @param {string} contactId
   * @param {Object} queryParams
   * @return {Promise}
   */
  getContactHistory(directoryId, contactId, queryParams) {
    return this.http.get(uri._contactsHistoryUri(directoryId, contactId), queryParams);
  }

  /**
   * @param {string} directoryId
   * @param {Object} bodyParams
   * @return {Promise}
   */
  createDirectoryContact(directoryId, bodyParams) {
    return this.http.post(uri._contactsUri(directoryId, ''), bodyParams);
  }

  /**
   * @param {string} directoryId
   * @param {string} contactId
   * @param {Object} bodyParams
   * @return {Promise}
   */
  updateDirectoryContact(directoryId, contactId, bodyParams) {
    return this.http.put(uri._contactsUri(directoryId, contactId), bodyParams);
  }

  /**
   * @param {string} directoryId
   * @param {string} contactId
   * @return {Promise}
   */
  deleteDirectoryContact(directoryId, contactId) {
    return this.http.delete(uri._contactsUri(directoryId, contactId));
  }

  /**
   * @param {string} directoryId
   * @param {Object} bodyParams
   * @return {Promise}
   */
  getFilteredContacts(directoryId, bodyParams) {
    return this.http.post(`${uri._directoriesUri(directoryId)}/filteredcontacts`, bodyParams);
  }
}

module.exports = Contacts;
