const url = require('url');
const autoBind = require('../util/autobind');

/**
 * MailingLists
 */
class MailingLists {
  constructor(http) {
    this.http = http;
    autoBind(this);
  }

  /**
   * @param {Object} queryParams
   * @return {Promise}
   */
  listMailingLists(queryParams) {
    return this.http.get(_mailingListsUri(''), queryParams);
  }

  /**
   * @param {string} mailingListId
   * @param {Object} queryParams
   * @return {Promise}
   */
  getMailingList(mailingListId) {
    return this.http.get(_mailingListsUri(mailingListId));
  }

  /**
   * @param {Object} bodyParams
   * @return {Promise}
   */
  createMailingList(bodyParams) {
    return this.http.post(_mailingListsUri(''), bodyParams);
  }

  /**
   * @param {string} mailingListId
   * @param {Object} bodyParams
   * @return {Promise}
   */
  updateMailingList(mailingListId, bodyParams) {
    return this.http.put(_mailingListsUri(mailingListId), bodyParams);
  }

  /**
   * @param {string} mailingListId
   * @return {Promise}
   */
  deleteMailingList(mailingListId) {
    return this.http.delete(_mailingListsUri(mailingListId));
  }

  /**
   * @param {string} mailingListId
   * @param {Object} queryParams
   * @return {Promise}
   */
  listContacts(mailingListId, queryParams) {
    return this.http.get(_mailingListsContactsUri(mailingListId, ''), queryParams);
  }

  /**
   * @param {string} mailingListId
   * @param {Object} queryParams
   * @param {Array} contacts
   * @return {Promise}
   */
  listAllContacts(mailingListId, queryParams = {}, contacts = []) {
    return this.listContacts(mailingListId, queryParams)
      .then((response) => {
        contacts = contacts.concat(response.result.elements);
        if (response.result.nextPage) {
          let nextPage = url.parse(response.result.nextPage, true);
          queryParams.skipToken = nextPage.query.skipToken;
          return this.listAllContacts(mailingListId, queryParams, contacts);
        }
        return contacts;
      });
  }

  /**
   * @param {string} mailingListId
   * @param {string} contactId
   * @return {Promise}
   */
  getContact(mailingListId, contactId) {
    return this.http.get(_mailingListsContactsUri(mailingListId, contactId));
  }

  /**
   * @param {string} mailingListId
   * @param {Object} bodyParams
   * @return {Promise}
   */
  createContact(mailingListId, bodyParams) {
    return this.http.post(_mailingListsContactsUri(mailingListId, ''), bodyParams);
  }

  /**
   * @param {string} mailingListId
   * @param {string} contactId
   * @param {Object} bodyParams
   * @return {Promise}
   */
  updateContact(mailingListId, contactId, bodyParams) {
    return this.http.put(_mailingListsContactsUri(mailingListId, contactId), bodyParams);
  }

  /**
   * @param {string} mailingListId
   * @param {string} contactId
   * @return {Promise}
   */
  deleteContact(mailingListId, contactId) {
    return this.http.delete(_mailingListsContactsUri(mailingListId, contactId));
  }

  /**
   * @param {string} mailingListId
   * @param {Object} headers
   * @param {Object} bodyParams
   * @return {Promise}
   */
  createContactsImport(mailingListId, headers, bodyParams) {
    return this.http.fetchJsonWithAuth(_mailingListsContactImportsUri(mailingListId, ''), {
      method: 'POST',
      headers: headers,
      body: JSON.stringify(bodyParams),
    });
  }

  /**
   * @param {string} mailingListId
   * @param {string} importId
   * @return {Promise}
   */
  getContactsImportProgress(mailingListId, importId) {
    return this.http.get(_mailingListsContactImportsUri(mailingListId, importId));
  }

  /**
   * @param {string} mailingListId
   * @return {Promise}
   */
  listSamples(mailingListId) {
    return this.http.get(_mailingListsSamplesUri(mailingListId, ''));
  }

  /**
   * @param {string} mailingListId
   * @param {string} sampleId
   * @return {Promise}
   */
  getSample(mailingListId, sampleId) {
    return this.http.get(_mailingListsSamplesUri(mailingListId, sampleId));
  }

  /**
   * @param {string} mailingListId
   * @param {string} contactId
   * @return {Promise}
   */
  getContactIncentivePoints(mailingListId, contactId) {
    return this.http.get(_mailingListsContactsIncentivePointsUri(mailingListId, contactId));
  }

  /**
   * @param {string} mailingListId
   * @param {string} contactId
   * @param {Object} bodyParams
   * @return {Promise}
   */
  incrementContactIncentivePoints(mailingListId, contactId, bodyParams) {
    return this.http.post(_mailingListsContactsIncentivePointsIncrementUri(mailingListId, contactId), bodyParams);
  }

  /**
   * @param {string} mailingListId
   * @param {string} importId
   * @return {Promise}
   */
  getContactsImportProgressSummary(mailingListId, importId) {
    return this.http.get(_mailingListsContactImportsSummaryUri(mailingListId, importId));
  }
}

function _mailingListsUri(mailingListId = '') {
  return `/mailinglists/${mailingListId}`;
}

function _mailingListsContactsUri(mailingListId, contactId = '') {
  return `${_mailingListsUri(mailingListId)}/contacts/${contactId}`;
}

function _mailingListsContactImportsUri(mailingListId, importId = '') {
  return `${_mailingListsUri(mailingListId)}/contactimports/${importId}`;
}

function _mailingListsContactImportsSummaryUri(mailingListId, importId = '') {
  return `${_mailingListsUri(mailingListId)}/contactimports/${importId}/summary`;
}

function _mailingListsSamplesUri(mailingListId, sampleId = '') {
  return `${_mailingListsUri(mailingListId)}/samples/${sampleId}`;
}

function _mailingListsContactsIncentivePointsUri(mailingListId, contactId = '') {
  return `${_mailingListsUri(mailingListId)}/contacts/${contactId}/incentivepoints`;
}

function _mailingListsContactsIncentivePointsIncrementUri(mailingListId, contactId = '') {
  return `${_mailingListsUri(mailingListId)}/contacts/${contactId}/incentivepoints/actions/increment`;
}

module.exports = MailingLists;
