const fs = require('fs');
const os = require('os');
const parseJsonFile = require('../util/parseJsonFile');
const path = require('path');
const promisify = require('../util/promisify');
const unzipFile = require('../util/unzipFile');
const pollUntilFinished = require('../util/pollUntilFinished');
const unlink = promisify(fs.unlink);
const autoBind = require('../util/autobind');

/**
 * ResponseExports
 */
class ResponseExports {
  constructor(http) {
    this.http = http;
    autoBind(this);
  }

  /**
   * @param {Object} bodyParams
   * @return {Promise}
   */
  getResponses(bodyParams = {}) {
    bodyParams.format = 'json';
    let filePathToDelete;
    return this.getResponsesFile(bodyParams)
      .then((filePath) => {
        filePathToDelete = filePath;
        return parseJsonFile(filePath);
      })
      .then((responses) => {
        unlink(filePathToDelete);
        return responses['responses'];
      });
  }

  /**
   * @param {Object} bodyParams
   * @return {Promise}
   */
  getResponsesFile(bodyParams) {
    let pathToDelete;
    return this.createResponseExport(bodyParams)
      .then((response) => response.result.id)
      .then((id) => pollUntilFinished(id, this.getResponseExportProgress))
      .then(this.getResponseExportFile)
      .then((zipFilePath) => pathToDelete = zipFilePath)
      .then(unzipFile)
      .then((filePath) => {
        unlink(pathToDelete);
        return filePath;
      });
  }

  /**
   * @param {Object} bodyParams
   * @return {Promise}
   */
  createResponseExport(bodyParams) {
    return this.http.post(_responseExportsUri(), bodyParams);
  }

  /**
   * @param {string} responseExportId
   * @return {Promise}
   */
  getResponseExportProgress(responseExportId) {
    return this.http.get(_responseExportsUri(responseExportId));
  }

  /**
   * @param {string} responseExportId
   * @return {Promise}
   */
  getResponseExportFile(responseExportId) {
    const filePath = path.join(os.tmpdir(), `${responseExportId}.zip`);
    return this.http.fetchWithAuth(_responseExportsUri(responseExportId) + '/file')
      .then((response) => {
        const fileStream = fs.createWriteStream(filePath);
        return new Promise((resolve, reject) => {
          response.body
            .on('error', reject)
            .pipe(fileStream);
          fileStream
            .on('error', reject)
            .on('finish', () => resolve(filePath));
        });
      });
  }
}

function _responseExportsUri(responseExportId = '') {
  return `/responseexports/${responseExportId}`;
}

module.exports = ResponseExports;
