const FormData = require('form-data');
const fs = require('fs');
const querystring = require('querystring');
const autoBind = require('../util/autobind');

/**
 * Surveys
 */
class Surveys {
  constructor(http) {
    this.http = http;
    autoBind(this);
  }

  /**
   * @param {number} offset
   * @param {Array} surveys
   * @return {Promise}
   */
  listAllSurveys(offset = 0, surveys = []) {
    return this.listSurveys({offset})
      .then((response) => {
        surveys = surveys.concat(response.result.elements);
        if (response.result.nextPage) {
          const queryParams = querystring.parse(
            response.result.nextPage.split('?')[1]);
          offset = queryParams.offset;
          return this.listAllSurveys(offset, surveys);
        }
        return surveys;
      });
  }

  /**
   * @param {Object} queryParams
   * @return {Promise}
   */
  listSurveys(queryParams) {
    return this.http.get(_surveysUri(), queryParams);
  }

  /**
   * @param {string} surveyId
   * @return {Promise}
   */
  getSurvey(surveyId) {
    return this.http.get(_surveysUri(surveyId));
  }

  /**
   * @param {string} surveyId
   * @return {Promise}
   */
  deleteSurvey(surveyId) {
    return this.http.delete(_surveysUri(surveyId));
  }

  /**
   * @param {string} surveyId
   * @param {Object} bodyParams
   * @return {Promise}
   */
  updateSurvey(surveyId, bodyParams) {
    return this.http.put(_surveysUri(surveyId), bodyParams);
  }

  /**
   * @param {string} filePath
   * @param {Object} bodyParams
   * @return {Promise}
   */
  importSurvey(filePath, bodyParams) {
    const formData = new FormData();
    formData.append(
      'file',
      fs.createReadStream(filePath),
      {contentType: 'application/vnd.qualtrics.survey.qsf'}
    );
    Object.keys(bodyParams).forEach((key) => {
      formData.append(key, bodyParams[key]);
    });
    const options = {
      method: 'POST',
      body: formData,
    };
    return this.http.fetchJsonWithAuth(_surveysUri(), options);
  }

  /**
   * @param {Object} bodyParams
   * @return {Promise}
   */
  importSurveyFromUrl(bodyParams) {
    return this.http.post(_surveysUri(), bodyParams);
  }

  /**
   * @param {string} xCopySource
   * @param {string} xCopyDestinationOwner
   * @param {Object} bodyParams
   * @return {Promise}
   */
  copySurvey(xCopySource, xCopyDestinationOwner, bodyParams) {
    return this.http.fetchJsonWithAuth(_surveysUri(), {
      headers: {
        'X-COPY-SOURCE': xCopySource,
        'X-COPY-DESTINATION-OWNER': xCopyDestinationOwner,
      },
      body: bodyParams,
    });
  }

  /**
   * @param {string} surveyId
   * @param {Object} bodyParams
   * @return {Promise}
   */
  shareSurvey(surveyId, bodyParams) {
    return this.http.post(`${_surveysUri(surveyId)}/permissions/collaborations`, bodyParams);
  }

  /**
   * @param {string} surveyId
   * @param {Object} bodyParams
   * @return {Promise}
   */
  insertEmbeddedDataFields(surveyId, bodyParams) {
    return this.http.post(`${_surveysUri(surveyId)}/embeddeddatafields`, bodyParams);
  }
}

function _surveysUri(surveyId = '') {
  return `/surveys/${surveyId}`;
}

module.exports = Surveys;
